using System;
using System.Diagnostics;
using System.IO;
using Terraria;
using TAPI;

namespace Autosave {
public class Autosave : ModBase
{
    public static bool saveClient = false;
    public static int savesSinceBackup = 0;
    private Stopwatch dedServSaveTimer = new Stopwatch();

    public override void PreGameUpdate()
    {
        int interval = (int)options["interval"].Value;
        if(Main.dedServ)
        {
            if((bool)options["serverSave"].Value)
            {
                if(!dedServSaveTimer.IsRunning)
                {
                    dedServSaveTimer.Start();
                }
                if(dedServSaveTimer.ElapsedMilliseconds > interval * 1000L)
                {
                    dedServSaveTimer.Reset();
                    StartAutosave();
                }
            }
        }
        else if(!Main.gameMenu)
        {
            if(Main.netMode == 1)
            {
                if(saveClient)
                {
                    Main.saveTime.Reset();
                    WorldGen.saveToonWhilePlaying();
                    saveClient = false;
                }
            }
            else if(Main.autoSave)
            {
                if(Main.saveTime.IsRunning && Main.saveTime.ElapsedMilliseconds > interval * 1000L)
                {
                    Main.saveTime.Reset();
                    if((bool)options["notifySave"].Value)
                    {
                        Main.NewText("<Autosave> Saving...", 100, 100, 255);
                    }
                    WorldGen.saveToonWhilePlaying();
                    StartAutosave();
                }
            }
        }
        else
        {
            savesSinceBackup = 0;
        }
    }

    public void StartAutosave()
    {
        System.Threading.ThreadPool.QueueUserWorkItem(new System.Threading.WaitCallback(AutosaveCallback), 1);
    }

    public void AutosaveCallback(object threadContext)
    {
        if((bool)options["backup"].Value)
        {
            savesSinceBackup++;
            if(savesSinceBackup >= (int)options["backupInterval"].Value)
            {
                BackupWorld();
                savesSinceBackup = 0;
            }
        }
        WorldGen.saveAndPlay();
        AutosaveNet.SendSaveToClients(this);
    }

    public void BackupWorld()
    {
        while(WorldGen.saveLock) {}
        WorldGen.saveLock = true;
        while(WorldGen.hardLock) {}
        if(!Main.skipMenu)
        {
            string path = Main.worldPathName;
            if(WorldGen.moveToWorldSelect)
            {
                path = Main.worldPathNameDest;
            }
            if(path != null && File.Exists(path))
            {
                if((bool)options["notifyBackup"].Value)
                {
                    if(Main.netMode == 2)
                    {
                        NetMessage.SendData(25, -1, -1, "<Autosave> Saving backup...", 255, 100, 100, 255);
                    }
                    else
                    {
                        Main.NewText("<Autosave> Saving backup...", 100, 100, 255);
                    }
                }
                string time = DateTime.Now.ToString().Replace(Path.DirectorySeparatorChar, '-');
                time = time.Replace('/', '-').Replace(':', '-');
                string dir = path + "_Backups" + Path.DirectorySeparatorChar;
                Directory.CreateDirectory(dir);
                string[] files = Directory.GetFiles(dir, "*.wld");
                if(files.Length >= (int)options["numBackups"].Value)
                {
                    string oldest = files[0];
                    for(int k = 1; k < files.Length; k++)
                    {
                        if(File.GetCreationTime(oldest).CompareTo(File.GetCreationTime(files[k])) > 0)
                        {
                            oldest = files[k];
                        }
                    }
                    File.Delete(oldest);
                }
                string dest = dir + time + ".wld";
                File.Copy(path, dest, true);
            }
        }
        WorldGen.saveLock = false;
    }
}}